import logging
from .utils.s3_utils import *
from .utils.ras_hdf import *
from .utils.ras_stac import *

from dotenv import find_dotenv, load_dotenv
import numpy as np
from .utils.common import check_params, GEOM_HDF_IGNORE_PROPERTIES
import logging
import sys
from papipyplug import parse_input, plugin_logger, print_results
from dotenv import load_dotenv, find_dotenv

logging.getLogger("boto3").setLevel(logging.WARNING)
logging.getLogger("botocore").setLevel(logging.WARNING)

# Set up logging
logging.basicConfig(
    level=logging.INFO,
    format="""{"time": "%(asctime)s" , "level": "%(levelname)s", "message": "%(message)s"}""",
    handlers=[logging.StreamHandler()],
)

load_dotenv(find_dotenv())


def new_geom_item(
    geom_hdf: str,
    new_item_s3_key: str,
    topo_assets: list = None,
    lulc_assets: list = None,
    mannings_assets: list = None,
    other_assets: list = None,
    item_props_to_remove: List = None,
    item_props_to_add: dict = None,
    minio_mode=False,
):
    verify_safe_prefix(new_item_s3_key)
    logging.info(f"Creating geom item: {new_item_s3_key}")
    item_public_url = s3_key_public_url_converter(new_item_s3_key, minio_mode=minio_mode)
    logging.debug(f"item_public_url: {item_public_url}")

    # Prep parameters
    bucket_name, _ = split_s3_key(geom_hdf)
    other_assets.append(geom_hdf)

    _, s3_client, s3_resource = init_s3_resources(minio_mode=minio_mode)
    bucket = s3_resource.Bucket(bucket_name)
    # Create geometry item
    if item_props_to_remove:
        item = create_model_item(geom_hdf, item_props_to_remove, minio_mode=minio_mode)
    else:
        item = create_model_item(geom_hdf, GEOM_HDF_IGNORE_PROPERTIES, minio_mode=minio_mode)

    if item_props_to_add:
        item.properties.update(item_props_to_add)
    # Create list of assets to add to item
    geom_assets = new_geom_assets(
        topo_assets=topo_assets,
        lulc_assets=lulc_assets,
        mannings_assets=mannings_assets,
        other_assets=other_assets,
    )

    # Add assets to item
    for asset_type, asset_list in geom_assets.items():
        logging.debug(asset_type)
        for asset_file in asset_list:
            _, asset_key = split_s3_key(asset_file)
            logging.info(f"Adding asset {asset_file} to item")
            obj = bucket.Object(asset_key)
            try:
                metadata = get_basic_object_metadata(obj)
            except:
                logging.error(f"unable to fetch metadata for :{obj}")
                metadata = {}
            asset_info = ras_geom_asset_info(asset_file, asset_type)
            asset = pystac.Asset(
                s3_key_public_url_converter(asset_file, minio_mode=minio_mode),
                extra_fields=metadata,
                roles=asset_info["roles"],
                description=asset_info["description"],
            )
            item.add_asset(asset_info["title"], asset)

    # Transform cell size properties to square root of area
    for prop in [
        "2d_flow_areas:cell_average_size",
        "2d_flow_areas:cell_maximum_size",
        "2d_flow_areas:cell_minimum_size",
    ]:
        try:
            item.properties[prop] = int(np.sqrt(float(item.properties[prop])))
        except KeyError:
            logging.warning(f"property {prop} not found")

    logging.info("Writing geom item to s3")
    item.set_self_href(item_public_url)
    copy_item_to_s3(item, new_item_s3_key, s3_client)

    logging.info("Program completed successfully")

    results = [
        {
            "href": item_public_url,
            "rel": "self",
            "title": "public_url",
            "type": "application/json",
        },
        {
            "href": new_item_s3_key,
            "rel": "self",
            "title": "s3_key",
            "type": "application/json",
        },
    ]

    return results


def main(params: dict, minio_mode=False):
    # Required parameters
    geom_hdf = params.get("geom_hdf", None)
    item_s3_key = params.get("new_item_s3_key", None)

    # Optional parameters
    topo_assets = params.get("topo_assets", [])
    lulc_assets = params.get("lulc_assets", [])
    mannings_assets = params.get("mannings_assets", [])
    other_assets = params.get("other_assets", [])
    item_props_to_remove = params.get("item_props_to_remove", [])
    item_props_to_add = params.get("item_props", {})

    return new_geom_item(
        geom_hdf,
        item_s3_key,
        topo_assets,
        lulc_assets,
        mannings_assets,
        other_assets,
        item_props_to_remove,
        item_props_to_add,
        minio_mode,
    )


if __name__ == "__main__":
    plugin_logger()

    if not load_dotenv(find_dotenv()):
        logging.warning("No local .env found")

    PLUGIN_PARAMS = check_params(new_geom_item)
    input_params = parse_input(sys.argv, PLUGIN_PARAMS)
    result = main(input_params, minio_mode=True)
    print_results(result)



from typing import List, Any
import inspect


def check_params(func):
    sig = inspect.signature(func)
    args = sig.parameters

    required = [name for name, param in args.items() if param.default == inspect.Parameter.empty]
    optional = [name for name, param in args.items() if param.default != inspect.Parameter.empty]

    return {"required": required, "optional": optional}


def get_dict_values(dicts: List[dict], key: Any) -> list:
    """
    This function retrieves the values of a specific key from a list of dictionaries.

    Parameters:
        dicts (List[dict]): The list of dictionaries.
        key (Any): The key to retrieve the values of.

    Returns:
        List[dict]: A list with the values of the key in the dictionaries. If a dictionary does not have the key, it is skipped.
    """
    results = []
    for d in dicts:
        if key in d:
            results.append(d[key])
    return results


GEOM_HDF_IGNORE_PROPERTIES = [
    "geometry:complete_geometry",
    "2d_flow_areas:cell_volume_tolerance",
    "2d_flow_areas:data_date",
    "2d_flow_areas:extents",
    "2d_flow_areas:cell_maximum_index",
    "2d_flow_areas:face_area_conveyance_ratio",
    "2d_flow_areas:face_area_elevation_tolerance",
    "2d_flow_areas:face_profile_tolerance",
    "2d_flow_areas:infiltration_date_last_modified",
    "2d_flow_areas:infiltration_file_date",
    "2d_flow_areas:infiltration_filename",
    "2d_flow_areas:infiltration_layername",
    "2d_flow_areas:land_cover_date_last_modified",
    "2d_flow_areas:land_cover_file_date",
    "2d_flow_areas:land_cover_filename",
    "2d_flow_areas:land_cover_layername",
    "2d_flow_areas:mannings_n",
    "2d_flow_areas:property_tables_last_computed",
    "2d_flow_areas:terrain_file_date",
    "2d_flow_areas:terrain_filename",
    "2d_flow_areas:version",
    "2d_flow_areas:infiltration_override_table_hash",
    "2d_flow_areas:property_tables_lc_hash",
]

PLAN_HDF_IGNORE_PROPERTIES = [
    "plan_parameters:1d_cores",
    "plan_parameters:1d_methodology",
    "plan_parameters:1d2d_flow_tolerance",
    "plan_parameters:1d2d_maxiter",
    "plan_parameters:1d2d_minimum_flow_tolerance",
    "plan_parameters:1d2d_ws_tolerance",
    "plan_information:plan_shortid",
    "plan_information:plan_title",
    "plan_information:time_window",
    "plan_parameters:2d_boundary_condition_ramp_up_fraction",
    "plan_parameters:2d_boundary_condition_volume_check",
    "plan_parameters:2d_cores_per_mesh",
    "plan_parameters:2d_coriolis",
    "plan_parameters:2d_initial_conditions_ramp_up_time_hrs",
    "plan_parameters:2d_latitude_for_coriolis",
    "plan_parameters:2d_longitudinal_mixing_coefficient",
    "plan_parameters:2d_matrix_solver",
    "plan_parameters:2d_maximum_iterations",
    "plan_parameters:2d_number_of_time_slices",
    "plan_parameters:2d_only",
    "plan_parameters:2d_smagorinsky_mixing_coefficient",
    "plan_parameters:2d_theta",
    "plan_parameters:2d_theta_warmup",
    "plan_parameters:2d_transverse_mixing_coefficient",
    "plan_parameters:2d_turbulence_formulation",
    "plan_parameters:gravity",
    "plan_information:flow_title",
    "plan_information:geometry_title",
    "plan_information:plan_title",
    "plan_parameters:hdf_chunk_size",
    "plan_parameters:hdf_compression",
    "plan_parameters:hdf_fixed_rows",
    "plan_parameters:hdf_flush_buffer",
    "plan_parameters:hdf_spatial_parts",
    "plan_parameters:hdf_use_max_rows",
    "plan_parameters:hdf_write_time_slices",
    "plan_parameters:hdf_write_warmup",
    "plan_parameters:pardiso_solver",
    "meteorology:enabled",
    "meteorology:raster_cols",
    "meteorology:raster_left",
    "meteorology:raster_rows",
    "meteorology:raster_top",
    "results_summary:run_time_window",
    "results_summary:computation_time_total_minutes",
    "unsteady_results:short_id",
    "volume_accounting:total_boundary_flux_of_water_in",
    "volume_accounting:total_boundary_flux_of_water_out",
    "volume_accounting:vol_accounting_in",
    "volume_accounting:error",
    "volume_accounting:volume_ending",
    "volume_accounting:volume_starting",
    "volume_accounting:precipitation_excess_acre_feet",
    "start_datetime",
    "end_datetime",
    "datetime",
]



import boto3
from dotenv import load_dotenv
import pystac
from pathlib import Path
import json
from datetime import datetime
from typing import Tuple, Dict
import shapely
from shapely.geometry import shape, box
import shapely.ops
import rasterio
from rasterio.session import AWSSession
import rasterio.warp
from dataclasses import dataclass
from mypy_boto3_s3.service_resource import Object
from rasterio.session import AWSSession
from .s3_utils import *
from rasterio.session import AWSSession
import logging

logging.getLogger("boto3").setLevel(logging.WARNING)
logging.getLogger("botocore").setLevel(logging.WARNING)


def get_raster_bounds(
    s3_key: str, aws_session: AWSSession, minio_mode: bool = False
) -> Tuple[float, float, float, float]:
    """
    This function retrieves the geographic bounds of a raster file stored in an AWS S3 bucket and returns them in the WGS 84 (EPSG:4326) coordinate reference system.

    Parameters:
        s3_key (str): The key of the raster file in the S3 bucket.
        aws_session (AWSSession): The AWS session to use to access the S3 bucket.

    Returns:
        Tuple[float, float, float, float]: The geographic bounds of the raster file in the WGS 84 (EPSG:4326) coordinate reference system. The bounds are returned as a tuple of four floats: (west, south, east, north).
    """
    if minio_mode:
        with rasterio.open(s3_key.replace("s3://", f"/vsicurl/{os.environ.get('MINIO_S3_ENDPOINT')}/")) as src:
            bounds = src.bounds
            crs = src.crs
            bounds_4326 = rasterio.warp.transform_bounds(crs, "EPSG:4326", *bounds)
            return bounds_4326

    else:
        with rasterio.Env(aws_session):
            with rasterio.open(s3_key) as src:
                bounds = src.bounds
                crs = src.crs
                bounds_4326 = rasterio.warp.transform_bounds(crs, "EPSG:4326", *bounds)
                return bounds_4326


def get_raster_metadata(s3_key: str, aws_session: AWSSession, minio_mode: bool = False) -> dict:
    """
    This function retrieves the metadata of a raster file stored in an AWS S3 bucket.

    Parameters:
        s3_key (str): The key of the raster file in the S3 bucket.
        aws_session (AWSSession): The AWS session to use to access the S3 bucket.

    Returns:
        dict: The metadata of the raster file. The metadata is returned as a dictionary
        where the keys are the names of the metadata items and the values are the values of the metadata items.
    """
    if minio_mode:
        with rasterio.open(s3_key.replace("s3://", f"/vsicurl/{os.environ.get('MINIO_S3_ENDPOINT')}/")) as src:
            return src.tags(1)
    else:
        with rasterio.Env(aws_session):
            with rasterio.open(s3_key) as src:
                return src.tags(1)


def bbox_to_polygon(bbox) -> shapely.Polygon:
    """
    This function converts a bounding box to a Shapely Polygon.

    Parameters:
        bbox: The bounding box to convert. It should be a sequence of four numbers: (min_x, min_y, max_x, max_y).

    Returns:
        shapely.Polygon: The Shapely Polygon representing the bounding box. The Polygon is a rectangle with the lower
          left corner at (min_x, min_y) and the upper right corner at (max_x, max_y).
    """
    min_x, min_y, max_x, max_y = bbox
    return shapely.Polygon(
        [
            [min_x, min_y],
            [min_x, max_y],
            [max_x, max_y],
            [max_x, min_y],
        ]
    )


def create_depth_grid_item(
    s3_obj: Object, item_id: str, aws_session: AWSSession, minio_mode: bool = False
) -> pystac.Item:
    """
    This function creates a PySTAC Item for a depth grid raster file stored in an AWS S3 bucket.

    Parameters:
        s3_obj (Object): The s3 object of the raster file in the S3 bucket.
        item_id (str): The ID to assign to the PySTAC Item.
        aws_session (AWSSession): The AWS session to use to access the S3 bucket.

    Returns:
        pystac.Item: The PySTAC Item representing the raster file. The Item has an Asset with the href set to the S3
        key of the raster file, the title set to the name of the raster file, the media type set to COG, and the role
        set to "ras-depth-grid". The Asset's extra fields are updated with the basic object metadata of the raster file
        and the metadata of the raster file. The Item's bbox is set to the geographic bounds of the raster file in the
        WGS 84 (EPSG:4326) coordinate reference system, the datetime is set to the current datetime, and the geometry
        is set to the GeoJSON representation of the bbox.
    """
    s3_full_key = f"s3://{s3_obj.bucket_name}/{s3_obj.key}"
    title = Path(s3_obj.key).name
    bbox = get_raster_bounds(s3_full_key, aws_session, minio_mode=minio_mode)
    geometry = bbox_to_polygon(bbox)
    item = pystac.Item(
        id=item_id,
        properties={},
        bbox=bbox,
        datetime=datetime.now(),
        geometry=json.loads(shapely.to_geojson(geometry)),
    )
    # non_null = not raster_is_all_null(depth_grid.key)
    asset = pystac.Asset(
        href=s3_key_public_url_converter(s3_full_key, minio_mode=minio_mode),
        title=title,
        media_type=pystac.MediaType.COG,
        roles=["ras-depth-grid"],
    )
    asset.extra_fields.update(get_basic_object_metadata(s3_obj))
    asset.extra_fields = dict(sorted(asset.extra_fields.items()))
    metadata = get_raster_metadata(s3_full_key, aws_session, minio_mode=minio_mode)
    if metadata:
        asset.extra_fields.update(metadata)
    item.add_asset(key=asset.title, asset=asset)
    return item



class RasStacInputs:
    """
    A class to generate input json files in the context of RAS STAC item generation.
    """

    def __init__(self, source_bucket_name: str, stac_bucket_name: str, **kwargs):
        self.source_bucket_name = source_bucket_name
        self.stac_bucket_name = stac_bucket_name

    def geometry_item(
        self,
        geom_hdf: str,
        new_item_s3_key: str,
        topo_assets: list,
        lulc_assets: list,
        mannings_assets: list,
        other_assets: list,
    ) -> dict:
        """
        Expects keys: "prefix/file.ext"
        """
        topo_assets = [f"s3://{self.source_bucket_name}/{a}" for a in topo_assets]
        lulc_assets = [f"s3://{self.source_bucket_name}/{a}" for a in lulc_assets]
        mannings_assets = [f"s3://{self.source_bucket_name}/{a}" for a in mannings_assets]
        other_assets = [f"s3://{self.source_bucket_name}/{a}" for a in other_assets]

        return {
            "geom_hdf": f"s3://{self.source_bucket_name}/{geom_hdf}",
            "new_item_s3_key": f"s3://{self.stac_bucket_name}/{new_item_s3_key}",
            "topo_assets": topo_assets,
            "mannings_assets": mannings_assets,
            "lulc_assets": lulc_assets,
            "other_assets": other_assets,
        }

    def plan_item(
        self,
        plan_hdf: str,
        new_plan_item_s3_key: str,
        geom_item_s3_key: str,
        sim_id: str,
        item_props: dict,
        ras_assets: list,
    ) -> dict:
        """
        Expects keys: "prefix/file.ext"
        """
        ras_assets = [f"s3://{self.source_bucket_name}/{a}" for a in ras_assets]

        return {
            "plan_hdf": f"s3://{self.source_bucket_name}/{plan_hdf}",
            "new_plan_item_s3_key": f"s3://{self.stac_bucket_name}/{new_plan_item_s3_key}",
            "geom_item_s3_key": f"s3://{self.stac_bucket_name}/{geom_item_s3_key}",
            "sim_id": sim_id,
            "item_props": item_props,
            "ras_assets": ras_assets,
        }

    def dg_item(
        self,
        plan_dg: str,
        new_dg_item_s3_key: str,
        plan_item_s3_key: str,
        dg_id: str,
        item_props: dict,
    ) -> dict:
        """
        Expects keys: "prefix/file.ext"
        """

        return {
            "plan_dg": f"s3://{self.source_bucket_name}/{plan_dg}",
            "new_dg_item_s3_key": f"s3://{self.stac_bucket_name}/{new_dg_item_s3_key}",
            "plan_item_s3_key": f"s3://{self.stac_bucket_name}/{plan_item_s3_key}",
            "dg_id": dg_id,
            "item_props": item_props,
        }



import pystac
import json
from typing import List
import os
import fsspec
from rashdf import RasPlanHdf, RasGeomHdf
from pathlib import Path
import re

import logging

logging.getLogger("boto3").setLevel(logging.WARNING)
logging.getLogger("botocore").setLevel(logging.WARNING)

from .ras_hdf import *


def create_model_item(
    ras_geom_hdf_url: str, props_to_remove: List, simplify: float = None, minio_mode: bool = False
) -> pystac.Item:
    """
    This function creates a STAC (SpatioTemporal Asset Catalog) item from a given HDF (Hierarchical Data Format)
    file URL.

    Parameters:
    ras_geom_hdf_url (str): The URL of the HDF file.
    props_to_remove (List): List of properties to be removed from item.

    Returns:
    pystac.Item: The created STAC item.

    Raises:
    ValueError: If the provided URL does not have a '.hdf' suffix.

    The function performs the following steps:
    1. Checks if the provided URL has a '.hdf' suffix. If not, it raises a ValueError.
    2. Extracts the model name from the URL by removing the '.hdf' suffix and getting the stem of the path.
    3. Logs the creation of the STAC item for the model.
    4. Opens the HDF file from the URL using the `RasGeomHdf.open_uri` method.
    5. Gets the perimeter of the 2D flow area from the HDF file and simplifies it using the provided `simplify`
      parameter.
    6. Gets the attributes of the geometry from the HDF file.
    7. Extracts the geometry time from the properties.
    8. Removes unwanted properties.
    9. Creates a new STAC item with the model ID, the geometry converted to GeoJSON, the bounding box of the perimeter, and the properties.
    10. Returns the created STAC item.
    """
    if Path(ras_geom_hdf_url).suffix != ".hdf":
        raise ValueError(f"Expected pattern: `s3://bucket/prefix/ras-model-name.g**.hdf`, got {ras_geom_hdf_url}")

    ras_model_name = Path(ras_geom_hdf_url.replace(".hdf", "")).stem

    logging.info(f"Creating STAC item for model {ras_model_name}")
    if minio_mode:
        ras_hdf = RasGeomHdf.open_uri(
            ras_geom_hdf_url, fsspec_kwargs={"endpoint_url": os.environ.get("MINIO_S3_ENDPOINT")}
        )
    else:
        ras_hdf = RasGeomHdf.open_uri(ras_geom_hdf_url)
    perimeter = ras_hdf.mesh_areas()
    perimeter = perimeter.to_crs("EPSG:4326")
    if simplify:
        perimeter_polygon = perimeter.geometry.unary_union.simplify(tolerance=simplify)
    else:
        perimeter_polygon = perimeter.geometry.unary_union
    properties = get_stac_geom_attrs(ras_hdf)
    if not properties:
        raise AttributeError(f"Could not find properties from: {ras_geom_hdf_url}")
    geometry_time = properties.get("geometry:geometry_time")

    # Remove unwanted properties
    for prop in props_to_remove:
        try:
            del properties[prop]
        except KeyError:
            logging.warning(f"Failed removing {prop}, property not found")

    model_id = ras_model_name

    item = pystac.Item(
        id=model_id,
        geometry=json.loads(shapely.to_geojson(perimeter_polygon)),
        bbox=perimeter_polygon.bounds,
        datetime=geometry_time,
        properties=properties,
    )
    return item


def new_geom_assets(
    topo_assets: list = None,
    lulc_assets: list = None,
    mannings_assets: list = None,
    other_assets: list = None,
):
    """
    This function creates a dictionary of geometric assets.

    Parameters:
        topo_assets (list): The topographic assets. Default is None.
        lulc_assets (list): The land use and land cover assets. Default is None.
        mannings_assets (list): The Manning's roughness coefficient assets. Default is None.
        other_assets (list): Any other assets. Default is None.

    Returns:
        dict: A dictionary with keys "topo", "lulc", "mannings", and "other", and values being the corresponding input
        parameters.
    """
    geom_assets = {
        "topo": topo_assets,
        "lulc": lulc_assets,
        "mannings": mannings_assets,
        "other": other_assets,
    }
    return geom_assets


def ras_geom_asset_info(s3_key: str, asset_type: str) -> dict:
    """
    This function generates information about a geometric asset used in a HEC-RAS model.

    Parameters:
        asset_type (str): The type of the asset. Must be one of: "mannings", "lulc", "topo", "other".
        s3_key (str): The S3 key of the asset.

    Returns:
        dict: A dictionary with the roles, the description, and the title of the asset.

    Raises:
        ValueError: If the provided asset_type is not one of: "mannings", "lulc", "topo", "other".
    """

    if asset_type not in ["mannings", "lulc", "topo", "other"]:
        raise ValueError(f"asset_type must be one of: mannings, lulc, topo, other")

    file_extension = Path(s3_key).suffix
    title = Path(s3_key).name

    if asset_type == "mannings":
        description = "Friction surface used in HEC-RAS model geometry"

    elif asset_type == "lulc":
        description = "Land Use / Land Cover data used in HEC-RAS model geometry"

    elif asset_type == "topo":
        description = "Topo data used in HEC-RAS model geometry"

    elif asset_type == "other":
        description = "Other data used in HEC-RAS model geometry"

    else:
        description = ""

    if file_extension == ".hdf":
        roles = [pystac.MediaType.HDF, f"ras-{asset_type}"]
    elif file_extension == ".tif":
        roles = [pystac.MediaType.GEOTIFF, f"ras-{asset_type}"]
    else:
        roles = [f"ras-{asset_type}"]

    return {"roles": roles, "description": description, "title": title}


def ras_plan_asset_info(s3_key: str) -> dict:
    """
    This function generates information about a plan asset used in a HEC-RAS model.

    Parameters:
        s3_key (str): The S3 key of the asset.

    Returns:
        dict: A dictionary with the roles, the description, and the title of the asset.

    The function performs the following steps:
    1. Extracts the file extension and the file name from the provided `s3_key`.
    2. If the file extension is ".hdf", it sets the `ras_extension` to the extension of the file name without the
      ".hdf" suffix and adds `pystac.MediaType.HDF5` to the roles. Otherwise, it sets the `ras_extension` to the file
        extension.
    3. Removes the leading dot from the `ras_extension`.
    4. Depending on the `ras_extension`, it sets the roles and the description for the asset. The `ras_extension` is
      expected to match one of the following patterns: "g[0-9]{2}", "p[0-9]{2}", "u[0-9]{2}", "s[0-9]{2}", "prj",
      "dss", "log". If it doesn't match any of these patterns, it adds "ras-file" to the roles.
    5. Returns a dictionary with the roles, the description, and the title of the asset.
    """
    file_extension = Path(s3_key).suffix
    title = Path(s3_key).name
    description = ""
    roles = []

    if file_extension == ".hdf":
        ras_extension = Path(s3_key.replace(".hdf", "")).suffix
        roles.append(pystac.MediaType.HDF5)
    else:
        ras_extension = file_extension

    ras_extension = ras_extension.lstrip(".")

    if re.match("g[0-9]{2}", ras_extension):
        roles.append("ras-geometry")
        if file_extension != ".hdf":
            roles.append(pystac.MediaType.TEXT)
            description = """The geometry file contains the 2D flow area perimeter and other geometry information."""

    elif re.match("p[0-9]{2}", ras_extension):
        roles.append("ras-plan")
        if file_extension != ".hdf":
            roles.append(pystac.MediaType.TEXT)
            description = """The plan file contains the simulation plan and other simulation information."""

    elif re.match("u[0-9]{2}", ras_extension):
        roles.extend(["ras-unsteady", pystac.MediaType.TEXT])
        description = """The unsteady file contains the unsteady flow results and other simulation information."""

    elif re.match("s[0-9]{2}", ras_extension):
        roles.extend(["ras-steady", pystac.MediaType.TEXT])
        description = """The steady file contains the steady flow results and other simulation information."""

    elif ras_extension == "prj":
        roles.extend(["ras-project", pystac.MediaType.TEXT])
        description = """The project file contains the project information and other simulation information."""

    elif ras_extension == "dss":
        roles.extend(["ras-dss"])
        description = """The dss file contains the dss results and other simulation information."""

    elif ras_extension == "log":
        roles.extend(["ras-log", pystac.MediaType.TEXT])
        description = """The log file contains the log information and other simulation information."""

    else:
        roles.extend(["ras-file"])

    return {"roles": roles, "description": description, "title": title}


def get_simulation_metadata(ras_plan_hdf_url: str, simulation: str, minio_mode: bool = False) -> dict:
    """
    This function retrieves the metadata of a simulation from a HEC-RAS plan HDF file.

    Parameters:
        ras_plan_hdf_url (str): The URL of the HEC-RAS plan HDF file.
        simulation (str): The name of the simulation.

    Returns:
        dict: A dictionary with the metadata of the simulation.

    The function performs the following steps:
    1. Opens the HEC-RAS plan HDF file from the provided URL in read-binary mode.
    2. Initializes a dictionary `metadata` with the key "ras:simulation" and the value being the provided
      `simulation` such as the plan id.
    3. Tries to open the HEC-RAS plan HDF file in read mode. If the file is not found, it logs an error
      and returns.
    4. Tries to get the plan attributes from the HDF file and update the `metadata` dictionary with them.
      If an exception occurs, it logs an error and returns.
    5. Tries to get the plan results attributes from the HDF file and update the `metadata` dictionary with
      them. If an exception occurs, it logs an error and returns.
    6. Returns the `metadata` dictionary.
    """
    if minio_mode:
        s3f = fsspec.open(
            ras_plan_hdf_url,
            client_kwargs={"endpoint_url": os.environ.get("MINIO_S3_ENDPOINT")},
            mode="rb",
        )
    else:
        s3f = fsspec.open(ras_plan_hdf_url, mode="rb")
    metadata = {
        "ras:simulation": simulation,
    }

    try:
        plan_hdf = RasPlanHdf(s3f.open())
    except FileNotFoundError as e:
        return logging.error(f"file not found: {ras_plan_hdf_url}")

    try:
        plan_attrs = get_stac_plan_attrs(plan_hdf)
        metadata.update(plan_attrs)
    except Exception as e:
        return logging.error(f"unable to extract plan_attrs from {ras_plan_hdf_url}: {e}")

    try:
        results_attrs = get_stac_plan_results_attrs(plan_hdf)
        metadata.update(results_attrs)
    except Exception as e:
        return logging.error(f"unable to extract results_attrs from {ras_plan_hdf_url}: {e}")

    return metadata


def create_model_simulation_item(
    ras_item: pystac.Item, results_meta: dict, model_sim_id: str, item_props_to_remove: List
) -> pystac.Item:
    """
    This function creates a PySTAC Item for a model simulation.

    Parameters:
        ras_item (pystac.Item): The PySTAC Item of the RAS model.
        results_meta (dict): The metadata of the simulation results.
        model_sim_id (str): The ID of the model simulation.
        item_props_to_remove (List): List of properties to be removed from the item.

    Returns:
        pystac.Item: A PySTAC Item for the model simulation.

    The function performs the following steps:
    1. Retrieves the runtime window from the `results_meta` dictionary.
    2. Removes unwanted properties.
    3. Creates a PySTAC Item with the ID being the `model_sim_id`, the geometry and the bounding box being those of
      the `ras_item`, the start and end datetimes being the converted start and end times of the runtime window,
      the datetime being the start datetime, and the properties being the `results_meta` with unwanted properties removed.
    4. Returns the created PySTAC Item.
    """
    runtime_window = results_meta.get("results_summary:run_time_window")
    start_datetime = runtime_window[0]
    end_datetime = runtime_window[1]

    for prop in item_props_to_remove:
        try:
            del results_meta[prop]
        except KeyError:
            logging.warning(f"Failed to remove property:{prop} not found in simulation results metadata.")

    item = pystac.Item(
        id=model_sim_id,
        geometry=ras_item.geometry,
        bbox=ras_item.bbox,
        start_datetime=start_datetime,
        end_datetime=end_datetime,
        datetime=start_datetime,
        properties=results_meta,
    )
    return item



import pyproj
import shapely
import shapely.ops
import logging
from dotenv import load_dotenv, find_dotenv

from rashdf import RasPlanHdf, RasGeomHdf
from rashdf.utils import parse_duration

logging.getLogger("boto3").setLevel(logging.WARNING)
logging.getLogger("botocore").setLevel(logging.WARNING)

load_dotenv(find_dotenv())


def to_snake_case(text):
    """
    Convert a string to snake case, removing punctuation and other symbols.

    Parameters:
        text (str): The string to be converted.

    Returns:
        str: The snake case version of the string.
    """
    import re

    # Remove all non-word characters (everything except numbers and letters)
    text = re.sub(r"[^\w\s]", "", text)

    # Replace all runs of whitespace with a single underscore
    text = re.sub(r"\s+", "_", text)

    return text.lower()


def prep_stac_attrs(attrs: dict, prefix: str = None) -> dict:
    """
    Converts an unformatted HDF attributes dictionary to STAC format by converting values to snake case
    and adding a prefix if one is given.

    Parameters:
        attrs (dict): Unformatted attribute dictionary.
        prefix (str): Optional prefix to be added to each key of formatted dictionary.

    Returns:
        results (dict): The new attribute dictionary snake case values and prefix.
    """
    results = {}
    for k, value in attrs.items():
        if prefix:
            key = f"{to_snake_case(prefix)}:{to_snake_case(k)}"
        else:
            key = k
        results[key] = value

    return results


def get_stac_plan_attrs(ras_hdf: RasPlanHdf, include_results: bool = False) -> dict:
    """
    This function retrieves the attributes of a plan from a HEC-RAS plan HDF file, converting them to STAC format.

    Parameters:
        ras_hdf (RasPlanHdf): An instance of RasPlanHdf which the attributes will be retrieved from.
        include_results (bool, optional): Whether to include the results attributes in the returned dictionary.
            Defaults to False.

    Returns:
        stac_plan_attrs (dict): A dictionary with the attributes of the plan.
    """
    stac_plan_attrs = ras_hdf.get_root_attrs()
    if stac_plan_attrs is not None:
        stac_plan_attrs = prep_stac_attrs(stac_plan_attrs)
    else:
        stac_plan_attrs = {}
        logging.warning("No root attributes found.")

    plan_info_attrs = ras_hdf.get_plan_info_attrs()
    if plan_info_attrs is not None:
        plan_info_stac_attrs = prep_stac_attrs(plan_info_attrs, prefix="Plan Information")
        stac_plan_attrs.update(plan_info_stac_attrs)
    else:
        logging.warning("No plan information attributes found.")

    plan_params_attrs = ras_hdf.get_plan_param_attrs()
    if plan_params_attrs is not None:
        plan_params_stac_attrs = prep_stac_attrs(plan_params_attrs, prefix="Plan Parameters")
        stac_plan_attrs.update(plan_params_stac_attrs)
    else:
        logging.warning("No plan parameters attributes found.")

    precip_attrs = ras_hdf.get_meteorology_precip_attrs()
    if precip_attrs is not None:
        precip_stac_attrs = prep_stac_attrs(precip_attrs, prefix="Meteorology")
        precip_stac_attrs.pop("meteorology:projection", None)
        stac_plan_attrs.update(precip_stac_attrs)
    else:
        logging.warning("No meteorology precipitation attributes found.")

    if include_results:
        stac_plan_attrs.update(get_stac_plan_results_attrs(ras_hdf))
    return stac_plan_attrs


def get_stac_plan_results_attrs(ras_hdf: RasPlanHdf):
    """
    This function retrieves the results attributes of a plan from a HEC-RAS plan HDF file, converting them to STAC format.
    For summary atrributes, it retrieves the total computation time, the run time window,
    and the solution from it, and calculates the total computation time in minutes if it exists.

    Parameters:
        ras_hdf (RasPlanHdf): An instance of RasPlanHdf which the results attributes will be retrieved from.

    Returns:
        results_attrs (dict): A dictionary with the results attributes of the plan.
    """
    results_attrs = {}

    unsteady_results_attrs = ras_hdf.get_results_unsteady_attrs()
    if unsteady_results_attrs is not None:
        unsteady_results_stac_attrs = prep_stac_attrs(unsteady_results_attrs, prefix="Unsteady Results")
        results_attrs.update(unsteady_results_stac_attrs)
    else:
        logging.warning("No unsteady results attributes found.")

    summary_attrs = ras_hdf.get_results_unsteady_summary_attrs()
    if summary_attrs is not None:
        summary_stac_attrs = prep_stac_attrs(summary_attrs, prefix="Results Summary")
        computation_time_total = summary_stac_attrs.get("results_summary:computation_time_total")
        results_summary = {
            "results_summary:computation_time_total": computation_time_total,
            "results_summary:run_time_window": summary_stac_attrs.get("results_summary:run_time_window"),
            "results_summary:solution": summary_stac_attrs.get("results_summary:solution"),
        }
        if computation_time_total is not None:
            computation_time_total_minutes = parse_duration(computation_time_total).total_seconds() / 60
            results_summary["results_summary:computation_time_total_minutes"] = computation_time_total_minutes
        results_attrs.update(results_summary)
    else:
        logging.warning("No unsteady results summary attributes found.")

    volume_accounting_attrs = ras_hdf.get_results_volume_accounting_attrs()
    if volume_accounting_attrs is not None:
        volume_accounting_stac_attrs = prep_stac_attrs(volume_accounting_attrs, prefix="Volume Accounting")
        results_attrs.update(volume_accounting_stac_attrs)
    else:
        logging.warning("No results volume accounting attributes found.")

    return results_attrs


def get_stac_geom_attrs(ras_hdf: RasGeomHdf):
    """
    This function retrieves the geometry attributes of a HEC-RAS HDF file, converting them to STAC format.

    Parameters:
        ras_hdf (RasGeomHdf): An instance of RasGeomHdf which the geometry attributes will be retrieved from.

    Returns:
        stac_geom_attrs (dict): A dictionary with the organized geometry attributes.
    """

    stac_geom_attrs = ras_hdf.get_root_attrs()
    if stac_geom_attrs is not None:
        stac_geom_attrs = prep_stac_attrs(stac_geom_attrs)
    else:
        stac_geom_attrs = {}
        logging.warning("No root attributes found.")

    geom_attrs = ras_hdf.get_geom_attrs()
    if geom_attrs is not None:
        geom_stac_attrs = prep_stac_attrs(geom_attrs, prefix="Geometry")
        stac_geom_attrs.update(geom_stac_attrs)
    else:
        logging.warning("No base geometry attributes found.")

    structures_attrs = ras_hdf.get_geom_structures_attrs()
    if structures_attrs is not None:
        structures_stac_attrs = prep_stac_attrs(structures_attrs, prefix="Structures")
        stac_geom_attrs.update(structures_stac_attrs)
    else:
        logging.warning("No geometry structures attributes found.")

    d2_flow_area_attrs = ras_hdf.get_geom_2d_flow_area_attrs()
    if d2_flow_area_attrs is not None:
        d2_flow_area_stac_attrs = prep_stac_attrs(d2_flow_area_attrs, prefix="2D Flow Areas")
        cell_average_size = d2_flow_area_stac_attrs.get("2d_flow_area:cell_average_size", None)
        if cell_average_size is not None:
            d2_flow_area_stac_attrs["2d_flow_area:cell_average_length"] = cell_average_size**0.5
        else:
            logging.warning("Unable to add cell average size to attributes.")
        stac_geom_attrs.update(d2_flow_area_stac_attrs)
    else:
        logging.warning("No flow area attributes found.")

    return stac_geom_attrs



import logging
import sys
from .utils.s3_utils import *
from .utils.ras_hdf import *
from .utils.ras_stac import *
from pathlib import Path
from rasterio.session import AWSSession
from dotenv import find_dotenv, load_dotenv
import numpy as np
from .utils.common import check_params, PLAN_HDF_IGNORE_PROPERTIES
from papipyplug import parse_input, plugin_logger, print_results

logging.getLogger("boto3").setLevel(logging.WARNING)
logging.getLogger("botocore").setLevel(logging.WARNING)

# Set up logging
logging.basicConfig(
    level=logging.INFO,
    format="""{"time": "%(asctime)s" , "level": "%(levelname)s", "message": "%(message)s"}""",
    handlers=[logging.StreamHandler()],
)


def new_plan_item(
    plan_hdf: str,
    new_plan_item_s3_key: str,
    geom_item_s3_key: str,
    sim_id: str,
    asset_list: list = None,
    item_props: dict = None,
    item_props_to_remove: List = None,
    minio_mode: bool = False,
):
    verify_safe_prefix(new_plan_item_s3_key)
    plan_item_public_url = s3_key_public_url_converter(new_plan_item_s3_key, minio_mode=minio_mode)
    geom_item_public_url = s3_key_public_url_converter(geom_item_s3_key, minio_mode=minio_mode)

    # Prep parameters
    bucket_name, _ = split_s3_key(plan_hdf)
    asset_list.append(plan_hdf)

    # Instantitate S3 resources
    session, s3_client, s3_resource = init_s3_resources(minio_mode)
    bucket = s3_resource.Bucket(bucket_name)
    AWS_SESSION = AWSSession(session)

    logging.info("Creating plan item")

    # Create geometry item
    geom_item = pystac.Item.from_file(geom_item_public_url)

    logging.info("fetching plan metadata")
    plan_meta = get_simulation_metadata(plan_hdf, sim_id, minio_mode=minio_mode)
    if plan_meta:
        try:
            logging.info("creating plan item")
            if item_props_to_remove:
                plan_item = create_model_simulation_item(geom_item, plan_meta, sim_id, item_props_to_remove)
            else:
                plan_item = create_model_simulation_item(geom_item, plan_meta, sim_id, PLAN_HDF_IGNORE_PROPERTIES)
        except TypeError:
            return logging.error(
                f"unable to retrieve model results with geom data from {geom_item_public_url} and metadata from {plan_hdf}. please verify plan was executed and results exist"
            )
    else:
        raise AttributeError(f"No simulation metadata retrieved from {plan_hdf}")

    plan_item.add_derived_from(geom_item)
    plan_item.properties.update(item_props)

    if asset_list:
        for asset_file in asset_list:
            _, asset_key = split_s3_key(asset_file)
            obj = bucket.Object(asset_key)
            metadata = get_basic_object_metadata(obj)
            asset_info = ras_plan_asset_info(asset_file)
            asset = pystac.Asset(
                s3_key_public_url_converter(asset_file, minio_mode=minio_mode),
                extra_fields=metadata,
                roles=asset_info["roles"],
                description=asset_info["description"],
            )
            plan_item.add_asset(asset_info["title"], asset)

    logging.info("Writing geom item to s3")
    plan_item.set_self_href(plan_item_public_url)
    copy_item_to_s3(plan_item, new_plan_item_s3_key, s3_client)

    logging.info("Program completed successfully")

    results = [
        {
            "href": plan_item_public_url,
            "rel": "self",
            "title": "public_url",
            "type": "application/json",
        },
        {
            "href": new_plan_item_s3_key,
            "rel": "self",
            "title": "s3_key",
            "type": "application/json",
        },
    ]

    return results


def main(params: dict, minio_mode=False):
    #  Required parameters
    plan_hdf = params.get("plan_hdf", None)
    sim_id = params.get("sim_id", None)
    plan_item_s3_key = params.get("new_plan_item_s3_key", None)
    geom_item_s3_key = params.get("geom_item_s3_key", None)

    # Optional parameters
    asset_list = params.get("asset_list", [])
    plan_item_props = params.get("item_props", {})
    item_props_to_remove = params.get("item_props_to_remove", [])

    return new_plan_item(
        plan_hdf,
        plan_item_s3_key,
        geom_item_s3_key,
        sim_id,
        asset_list,
        plan_item_props,
        item_props_to_remove,
        minio_mode,
    )


if __name__ == "__main__":
    plugin_logger()

    if not load_dotenv(find_dotenv()):
        logging.warning("No local .env found")

    PLUGIN_PARAMS = check_params(new_plan_item)
    input_params = parse_input(sys.argv, PLUGIN_PARAMS)
    result = main(input_params, minio_mode=True)
    print_results(result)



from mypy_boto3_s3.service_resource import ObjectSummary
import boto3
import botocore
import os
import json
from datetime import datetime
from json import JSONEncoder
import logging
from dotenv import load_dotenv, find_dotenv

logging.getLogger("boto3").setLevel(logging.WARNING)
logging.getLogger("botocore").setLevel(logging.WARNING)

load_dotenv(find_dotenv())


def get_basic_object_metadata(obj: ObjectSummary) -> dict:
    """
    This function retrieves basic metadata of an AWS S3 object.

    Parameters:
        obj (ObjectSummary): The AWS S3 object.

    Returns:
        dict: A dictionary with the size, ETag, last modified date, storage platform, region, and storage tier of the object.
    """
    try:
        _ = obj.load()
        return {
            "file:size": obj.content_length,
            "e_tag": obj.e_tag.strip('"'),
            "last_modified": obj.last_modified.isoformat(),
            "storage:platform": "AWS",
            "storage:region": obj.meta.client.meta.region_name,
            "storage:tier": obj.storage_class,
        }
    except botocore.exceptions.ClientError:
        raise KeyError(f"Unable to access {obj.key} check that key exists and you have access")


class DateTimeEncoder(JSONEncoder):
    """JSON encoder for handling datetime objects."""

    def default(self, obj):
        if isinstance(obj, datetime):
            return obj.isoformat()
        return JSONEncoder.default(self, obj)


def copy_item_to_s3(item, s3_key, s3client):
    """
    This function copies an item to an AWS S3 bucket.

    Parameters:
        item: The item to copy. It must have a `to_dict` method that returns a dictionary representation of it.
        s3_key (str): The file path in the S3 bucket to copy the item to.

    The function performs the following steps:
        1. Initializes a boto3 S3 client and splits the s3_key into the bucket name and the key.
        2. Converts the item to a dictionary, serializes it to a JSON string, and encodes it to bytes.
        3. Puts the encoded JSON string to the specified file path in the S3 bucket.
    """
    # s3 = boto3.client("s3")
    bucket, key = split_s3_key(s3_key)

    item_json = json.dumps(item.to_dict(), cls=DateTimeEncoder).encode("utf-8")

    s3client.put_object(Body=item_json, Bucket=bucket, Key=key)


def split_s3_key(s3_key: str) -> tuple[str, str]:
    """
    This function splits an S3 key into the bucket name and the key.

    Parameters:
        s3_key (str): The S3 key to split. It should be in the format 's3://bucket/key'.

    Returns:
        tuple: A tuple containing the bucket name and the key. If the S3 key does not contain a key, the second element
          of the tuple will be None.

    The function performs the following steps:
        1. Removes the 's3://' prefix from the S3 key.
        2. Splits the remaining string on the first '/' character.
        3. Returns the first part as the bucket name and the second part as the key. If there is no '/', the key will
          be None.
    """
    parts = s3_key.replace("s3://", "").split("/", 1)
    bucket = parts[0]
    key = parts[1] if len(parts) > 1 else None
    return bucket, key


def s3_key_public_url_converter(url: str, minio_mode: bool = False) -> str:
    """
    This function converts an S3 URL to an HTTPS URL and vice versa.

    Parameters:
        url (str): The URL to convert. It should be in the format 's3://bucket/' or 'https://bucket.s3.amazonaws.com/'.

    Returns:
        str: The converted URL. If the input URL is an S3 URL, the function returns an HTTPS URL. If the input URL is
        an HTTPS URL, the function returns an S3 URL.

    The function performs the following steps:
        1. Checks if the input URL is an S3 URL or an HTTPS URL.
        2. If the input URL is an S3 URL, it converts it to an HTTPS URL.
        3. If the input URL is an HTTPS URL, it converts it to an S3 URL.
    """

    if url.startswith("s3"):
        bucket = url.replace("s3://", "").split("/")[0]
        key = url.replace(f"s3://{bucket}", "")[1:]
        if minio_mode:
            logging.info(f"minio_mode | using minio endpoint for s3 url conversion: {url}")
            return f"{os.environ.get('MINIO_S3_ENDPOINT')}/{bucket}/{key}"
        else:
            return f"https://{bucket}.s3.amazonaws.com/{key}"

    elif url.startswith("http"):
        if minio_mode:
            logging.info(f"minio_mode | using minio endpoint for s3 url conversion: {url}")
            bucket = url.replace(os.environ.get("MINIO_S3_ENDPOINT"), "").split("/")[0]
            key = url.replace(os.environ.get("MINIO_S3_ENDPOINT"), "")
        else:
            bucket = url.replace("https://", "").split(".s3.amazonaws.com")[0]
            key = url.replace(f"https://{bucket}.s3.amazonaws.com/", "")

        return f"s3://{bucket}/{key}"

    else:
        raise ValueError(f"Invalid URL format: {url}")


def verify_safe_prefix(s3_key: str):
    """
    TODO: discuss this with the team. Would like some safety mechanism to ensure that the S3 key is limited to
    certain prefixes. Should there be some restriction where these files can be written?
    """
    parts = s3_key.split("/")
    logging.debug(f"parts of the s3_key: {parts}")
    if parts[3] != "stac":
        raise ValueError(f"prefix must begin with stac/, user provided {s3_key} needs to be corrected")


def init_s3_resources(minio_mode: bool = False):
    if minio_mode:
        session = boto3.Session(
            aws_access_key_id=os.environ.get("MINIO_ACCESS_KEY_ID"),
            aws_secret_access_key=os.environ.get("MINIO_SECRET_ACCESS_KEY"),
        )

        s3_client = session.client("s3", endpoint_url=os.environ.get("MINIO_S3_ENDPOINT"))

        s3_resource = session.resource("s3", endpoint_url=os.environ.get("MINIO_S3_ENDPOINT"))

        return session, s3_client, s3_resource
    else:
        # Instantitate S3 resources
        session = boto3.Session(
            aws_access_key_id=os.environ.get("AWS_ACCESS_KEY_ID"),
            aws_secret_access_key=os.environ.get("AWS_SECRET_ACCESS_KEY"),
        )

        s3_client = session.client("s3")
        s3_resource = session.resource("s3")
        return session, s3_client, s3_resource


def list_keys(s3_client, bucket, prefix, suffix=""):
    keys = []
    kwargs = {"Bucket": bucket, "Prefix": prefix}
    while True:
        resp = s3_client.list_objects_v2(**kwargs)
        keys += [obj["Key"] for obj in resp["Contents"] if obj["Key"].endswith(suffix)]
        try:
            kwargs["ContinuationToken"] = resp["NextContinuationToken"]
        except KeyError:
            break
    return keys


import re


def list_keys_regex(s3_client, bucket, prefix_includes, suffix=""):
    keys = []
    kwargs = {"Bucket": bucket, "Prefix": prefix_includes}
    prefix_pattern = re.compile(prefix_includes.replace("*", ".*"))
    while True:
        resp = s3_client.list_objects_v2(**kwargs)
        keys += [
            obj["Key"] for obj in resp["Contents"] if prefix_pattern.match(obj["Key"]) and obj["Key"].endswith(suffix)
        ]
        try:
            kwargs["ContinuationToken"] = resp["NextContinuationToken"]
        except KeyError:
            break
    return keys



import logging
import sys
from .utils.s3_utils import *
from .utils.dg_utils import *
from .utils.ras_stac import *
from pathlib import Path
from rasterio.session import AWSSession
from dotenv import find_dotenv, load_dotenv
import numpy as np
from .utils.common import check_params, PLAN_HDF_IGNORE_PROPERTIES
from papipyplug import parse_input, plugin_logger, print_results
from .utils.dg_utils import create_depth_grid_item

logging.getLogger("boto3").setLevel(logging.WARNING)
logging.getLogger("botocore").setLevel(logging.WARNING)

# Set up logging
logging.basicConfig(
    level=logging.INFO,
    format="""{"time": "%(asctime)s" , "level": "%(levelname)s", "message": "%(message)s"}""",
    handlers=[logging.StreamHandler()],
)


def new_plan_dg_item(
    plan_dg: str,
    new_dg_item_s3_key: str,
    plan_item_s3_key: str,
    dg_id: str,
    item_props: dict = None,
    asset_list: list = None,
    minio_mode: bool = False,
):
    logging.info("Creating plan item")
    verify_safe_prefix(new_dg_item_s3_key)

    dg_item_public_url = s3_key_public_url_converter(new_dg_item_s3_key, minio_mode=minio_mode)
    plan_item_public_url = s3_key_public_url_converter(plan_item_s3_key, minio_mode=minio_mode)

    # Prep parameters
    bucket_name, _ = split_s3_key(plan_dg)

    # Instantitate S3 resources
    session, s3_client, s3_resource = init_s3_resources(minio_mode)
    bucket = s3_resource.Bucket(bucket_name)
    AWS_SESSION = AWSSession(session)

    logging.info("pulling plan item")
    plan_item = pystac.Item.from_file(plan_item_public_url)

    _, key = split_s3_key(plan_dg)
    dg_obj = bucket.Object(key)

    logging.info("fetching dg metadata")
    dg_item = create_depth_grid_item(dg_obj, dg_id, AWS_SESSION, minio_mode=minio_mode)
    dg_item.properties.update(item_props)
    dg_item.add_derived_from(plan_item)

    for asset_file in asset_list:
        _, asset_key = split_s3_key(asset_file)
        obj = bucket.Object(asset_key)
        metadata = get_basic_object_metadata(obj)
        asset_info = ras_plan_asset_info(asset_file)
        asset = pystac.Asset(
            s3_key_public_url_converter(asset_file, minio_mode=minio_mode),
            extra_fields=metadata,
            roles=asset_info["roles"],
            description=asset_info["description"],
        )
        dg_item.add_asset(asset_info["title"], asset)

    logging.info("Writing dg item to s3")
    dg_item.set_self_href(dg_item_public_url)
    copy_item_to_s3(dg_item, new_dg_item_s3_key, s3_client)

    logging.info("Program completed successfully")

    results = [
        {
            "href": dg_item_public_url,
            "rel": "self",
            "title": "public_url",
            "type": "application/json",
        },
        {
            "href": new_dg_item_s3_key,
            "rel": "self",
            "title": "s3_key",
            "type": "application/json",
        },
    ]

    return results


def main(params: dict, minio_mode=False):
    # Required parameters
    plan_dg = params.get("plan_dg", None)
    dg_id = params.get("dg_id", None)
    dg_item_s3_key = params.get("new_dg_item_s3_key", None)
    plan_item_s3_key = params.get("plan_item_s3_key", None)

    # Optional parameters
    asset_list = params.get("assets", [])
    dg_item_props = params.get("item_props", {})

    return new_plan_dg_item(
        plan_dg,
        dg_item_s3_key,
        plan_item_s3_key,
        dg_id,
        dg_item_props,
        asset_list,
        minio_mode,
    )


if __name__ == "__main__":
    plugin_logger()

    if not load_dotenv(find_dotenv()):
        logging.warning("No local .env found")

    PLUGIN_PARAMS = check_params(new_plan_dg_item)
    input_params = parse_input(sys.argv, PLUGIN_PARAMS)
    result = main(input_params, minio_mode=True)
    print_results(result)
